package com.example.what_the_fantasy.ui.screens

import android.util.Log
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.selection.selectable
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.Search
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.ColorFilter
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.semantics.Role
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.example.what_the_fantasy.R
import com.example.what_the_fantasy.data.model.Quote
import com.example.what_the_fantasy.data.services.IServices
import com.example.what_the_fantasy.ui.components.NavBar
import com.example.what_the_fantasy.ui.components.QuoteLittle
import com.example.what_the_fantasy.ui.states.CurrentUserState
import com.example.what_the_fantasy.ui.states.SearchState
import com.example.what_the_fantasy.ui.viewModels.AuthUserViewModel
import com.example.what_the_fantasy.ui.viewModels.CurrentUserViewModel
import com.example.what_the_fantasy.ui.viewModels.SearchViewModel
import kotlinx.coroutines.delay


@Composable
fun SearchPage(
    navFavorite: () -> Unit,
    navAccueil: () -> Unit,
    navProfil: () -> Unit,
    navQuiz: () -> Unit,
    navQuote: (Int) -> Unit,
    navSearch: (String, String) -> Unit,
    type: String,
    search: String,
    currentUserVM : CurrentUserViewModel,
    currentUserState : CurrentUserState,
    searchVM : SearchViewModel,
    searchState : SearchState,
) {


    val textQuoteSearch = stringResource(id = R.string.QuoteSearch)
    val textTitleSearch = stringResource(id = R.string.TitleQuoteSearch)
    val textCharacterSearch = stringResource(id = R.string.character)
    val textSearch = stringResource(id = R.string.Search)

    val filtre = listOf("contenu", "personnage", "titre")
    //val filtre = listOf(textQuoteSearch, textCharacterSearch, textTitleSearch)
    val (newFiltre, onFiltreSelected) = remember { mutableStateOf(type) }

    val page = remember { mutableIntStateOf(1) }
    val quotes = remember { mutableStateListOf<Quote>() }
    val isLoading = remember { mutableStateOf(false) }
    val state = rememberLazyListState()

    LaunchedEffect(page.intValue, type, search) {
        if (!isLoading.value) {
            isLoading.value = true
            delay(500)
            val newQuotes = searchVM.search(type, search, page.intValue * 15)
            val uniqueQuotes = newQuotes.filterNot { new -> quotes.any { it.id == new.id } }
            if (uniqueQuotes.isNotEmpty()) {
                quotes.addAll(uniqueQuotes)
            }
            isLoading.value = false
        }
    }

    NavBar(
        onAccueil = true,
        navControllerFavorite = navFavorite,
        navControllerAccueil = navAccueil,
        navControllerProfil = navProfil,
        navControllerQuiz = navQuiz,
        navControllerSearch = { navSearch("contenu", "") },
        currentUserVM = currentUserVM,
        currentUserState = currentUserState,


    ) {
        Column(
            modifier = Modifier
                .fillMaxSize()
                .background(MaterialTheme.colorScheme.background)
        ) {
            Text(
                text = "▶ " + stringResource(R.string.TitleSearch) + " ◀",
                color = MaterialTheme.colorScheme.onBackground,
                fontSize = 24.sp,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(16.dp),
                textAlign = TextAlign.Center
            )
            Column(horizontalAlignment = Alignment.CenterHorizontally) {
                OutlinedTextField(
                    value = searchState.search,
                    onValueChange = {
                        searchVM.setSearch(it)
                                    },
                    textStyle = TextStyle(color = MaterialTheme.colorScheme.onBackground),
                    label = { Text(textSearch) },
                    modifier = Modifier
                        .padding(top = 2.dp)
                        .fillMaxWidth()
                        .height(70.dp),
                    keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text),
                    maxLines = 1,
                    shape = CircleShape,
                    trailingIcon = {
                        IconButton(
                            onClick = { navSearch(newFiltre, searchState.search) },
                            modifier = Modifier
                                .size(50.dp)
                                .clip(CircleShape)
                        ) {

                            Image(
                                painter = painterResource(id = R.drawable.search),
                                contentDescription = "search",
                                modifier = Modifier
                                    .size(30.dp),
                                colorFilter = ColorFilter.tint(MaterialTheme.colorScheme.primary)
                            )
                        }
                    }
                )
                Row(horizontalArrangement = Arrangement.SpaceAround) {
                    filtre.forEach { typeItem ->
                        Row(
                            Modifier
                                .height(56.dp)
                                .selectable(
                                    selected = (typeItem == newFiltre),
                                    onClick = { onFiltreSelected(typeItem) },
                                    role = Role.RadioButton
                                )
                                .padding(horizontal = 16.dp),
                            verticalAlignment = Alignment.CenterVertically
                        ) {
                            RadioButton(
                                selected = (typeItem == newFiltre),
                                onClick = null
                            )
                            Text(
                                text = typeItem,
                                style = MaterialTheme.typography.bodyLarge,
                                modifier = Modifier.padding(start = 16.dp),
                                color = MaterialTheme.colorScheme.onBackground
                            )
                        }
                    }
                }
            }
            LazyColumn(
                modifier = Modifier.weight(1f),
                state = state
            ) {
                items(quotes) { quote ->
                    Column(Modifier.clickable { navQuote(quote.id) }) {
                        if(quote.language == currentUserState.langage) { // affiche les suggestions de citation dans la bonne langue
                            QuoteLittle(quote)
                        }
                    }
                }
                item {
                    if (state.layoutInfo.visibleItemsInfo.lastOrNull()?.index == quotes.size - 1 && !isLoading.value) {
                        LaunchedEffect(Unit) {
                            page.intValue++
                        }
                        Box(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(16.dp),
                            contentAlignment = Alignment.Center
                        ) {
                            CircularProgressIndicator()
                        }
                    }
                }
            }
        }
    }
}
